/**************************************************/ 
/*                                                */ 
/* LcdTest.c                                      */ 
/* LCD interface implementation                   */ 
/*                                                */ 
/* Version : 1.3                                  */ 
/*                                                */ 
/*                                                */ 
/* By: J. Winpenny                                */ 
/*                                                */ 
/* Date: 19/7/1999                                */ 
/*                                                */ 
/*                                                */ 
/* Mode : HD44780 type LCD displays               */ 
/*                                                */ 
/* Notes : Requires "Delays.c"                    */ 
/*                                                */ 
/* This version is for 4 bit mode                 */ 
/*                                                */ 
/*                                                */ 
/**************************************************/ 
#include "lcd.h"

code uchar hex_code[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

/***********************************/ 
/* Setup the lcd device */ 
/***********************************/ 
void LCD_Setup(void) 
{ 
/* Reset the LCD */ 

    LCD_SEL = 0;

    delay_ms(30); /* Power up delay */ 
    LCD_FunctionMode(); 
    
    
    LCD_Write_8_Bit( system_set_4_bit ); /* This sequence resets the LCD */ 
    delay_ms(5); 
    LCD_Write_8_Bit( system_set_4_bit ); 
    delay_ms(5); 
    LCD_Write_8_Bit( system_set_4_bit ); 
    delay_ms(5); 
    
    LCD_Write_4_Bit( system_set_4_bit ); 
    delay_ms(2); 
    LCD_Write_4_Bit( display_off ); 
    delay_ms(2); 
    LCD_Write_4_Bit( entry_mode ); 
    delay_ms(2); 
    LCD_Write_4_Bit( display_on ); 
    delay_ms(2); 
    LCD_Write_4_Bit( set_dd_ram ); 
    delay_ms(2); 
    
    LCD_DataMode(); 
} 

/* END OF MAIN */ 


/***********************************/ 
/* Put LCD in Function Mode */ 
/***********************************/ 
void LCD_FunctionMode(void) 
{ 
    LCD_RS = 0;
    LCD_Delay(); 
} 
/***********************************/ 
/* Put LCD in Data Mode */ 
/***********************************/ 
void LCD_DataMode(void) 
{ 
    LCD_RS = 1;
    LCD_Delay(); 
} 
/***********************************/ 
/* Write a single byte to the LCD */ 
/* 8 Bit Mode */ 
/***********************************/ 
void LCD_Write_8_Bit(char d ) 
{ 
/* Write Mode */ 
    LCD_WR = 0;
    LCD_Delay(); 

/* Setup data */ 
    if ( d & 0x80 )   LCD_DATA_7 = 1;
    else              LCD_DATA_7 = 0;
    
    if ( d & 0x40 )   LCD_DATA_6 = 1;
    else              LCD_DATA_6 = 0;
    
    if ( d & 0x20 )   LCD_DATA_5 = 1;
    else              LCD_DATA_5 = 0;
    
    if ( d & 0x10 )   LCD_DATA_4 = 1;
    else              LCD_DATA_4 = 0;
    
    LCD_Delay(); 
    LCD_SEL = 1;
    LCD_Delay(); 
    LCD_SEL = 0;/* De-select LCD */ 
} 
/***********************************/ 
/* Write a single byte to the LCD */ 
/* 4 Bit Mode */ 
/***********************************/ 
void LCD_Write_4_Bit(char d ) 
{ 
/* Write Mode */ 
    LCD_WR = 0;
//    LCD_Delay(); 

/* Output Higher 4 bits */ 

    if ( d & 0x80 )    LCD_DATA_7 = 1;
    else               LCD_DATA_7 = 0;
    
    if ( d & 0x40 )    LCD_DATA_6 = 1;
    else               LCD_DATA_6 = 0;
    
    if ( d & 0x20 )    LCD_DATA_5 = 1;
    else               LCD_DATA_5 = 0;
    
    if ( d & 0x10 )    LCD_DATA_4 = 1;
    else               LCD_DATA_4 = 0;

    
//    LCD_Delay(); 
    LCD_SEL = 1;
//    LCD_Delay(); 
/* Clock in the data */ 
    LCD_SEL = 0;
//    LCD_Delay(); 
    LCD_Wait();
    
    LCD_WR = 0;
    LCD_RS = 1;
    
    d <<= 4; /* Output Lower 4 bits */ 
    
    
    if ( d & 0x80 )    LCD_DATA_7 = 1;
    else               LCD_DATA_7 = 0;
    
    if ( d & 0x40 )    LCD_DATA_6 = 1;
    else               LCD_DATA_6 = 0;
    
    if ( d & 0x20 )    LCD_DATA_5 = 1;
    else               LCD_DATA_5 = 0;
    
    if ( d & 0x10 )    LCD_DATA_4 = 1;
    else               LCD_DATA_4 = 0;

    
//    LCD_Delay(); 
    LCD_SEL = 1; 
//    LCD_Delay(); 
    LCD_SEL = 0; /* Clock in the data */ 

    LCD_Wait();
} 

/***********************************/ 
/* LCD timing delay */ 
/* Adjust for your LCD Display */ 
/***********************************/ 
void LCD_Delay(void) 
{ 
    delay_ms(1); 
} 
/***********************************/ 
//
/***********************************/ 
void delay_ms(unsigned char ms)
{
     xdata uint ui;
     ui = ms*10;
/*     if (ms == 1) 
        WaitResp();
     else */
        while (ui) ui--;
}
/***********************************/ 
//
/***********************************/ 
void PrintUintHex(uint ui, uchar Position)
{
    SetPos(Position);
    LCD_Write_4_Bit(hex_code[(ui >> 12)&0x0F]);
    LCD_Write_4_Bit(hex_code[(ui >>  8)&0x0F]);
    LCD_Write_4_Bit(hex_code[(ui >>  4)&0x0F]);
    LCD_Write_4_Bit(hex_code[(ui      )&0x0F]);
}
/***********************************/ 
//
/***********************************/ 
void PrintUint(uint ui, uchar Position)
{
    xdata uchar a[5];

    a[0] = 0; while (ui > 9999) { ui -= 10000; a[0]++;} a[0] |= 0x30; 
    a[1] = 0; while (ui >  999) { ui -=  1000; a[1]++;} a[1] |= 0x30; 
    a[2] = 0; while (ui >   99) { ui -=   100; a[2]++;} a[2] |= 0x30; 
    a[3] = 0; while (ui >    9) { ui -=    10; a[3]++;} a[3] |= 0x30; 
    a[4] = ui | 0x30;                                                 

    SetPos(Position);
    LCD_Write_4_Bit(a[0]); LCD_Write_4_Bit(a[1]); LCD_Write_4_Bit(a[2]); 
    LCD_Write_4_Bit(a[3]); LCD_Write_4_Bit(a[4]);
}
/***********************************/ 
//
/***********************************/ 
void PrintUcharHex(uchar uc, uchar Position)
{
    SetPos(Position);
    LCD_Write_4_Bit(hex_code[(uc >>  4)&0x0F]);
    LCD_Write_4_Bit(hex_code[(uc      )&0x0F]);
}
/***********************************/ 
//
/***********************************/ 
void PrintFloat(float fl, uchar Position)
{
    PrintUint(fl,Position);
/*    xdata uchar a[5], b[2];
    a[0] = 0; while (fl > 9999.0)   { fl -= 10000.0;  a[0]++; } a[0] |= 0x30; 
    a[1] = 0; while (fl >  999.0)   { fl -=  1000.0;  a[1]++; } a[1] |= 0x30; 
    a[2] = 0; while (fl >   99.0)   { fl -=   100.0;  a[2]++; } a[2] |= 0x30; 
    a[3] = 0; while (fl >    9.0)   { fl -=    10.0;  a[3]++; } a[3] |= 0x30; 
    a[4] = 0; while (fl >    0.9)   { fl -=     1.0;  a[4]++; } a[4] |= 0x30;                                                 

    b[0] = 0; while (fl >    0.09)  { fl -=     0.1;  b[0]++; } b[0] |= 0x30; 
    b[1] = 0; while (fl >    0.009) { fl -=     0.01; b[1]++; } b[1] |= 0x30; 

    SetPos(Position);
    LCD_Write_4_Bit(a[0]); LCD_Write_4_Bit(a[1]); 
    LCD_Write_4_Bit(a[2]); LCD_Write_4_Bit(a[3]); 
    LCD_Write_4_Bit(a[4]); LCD_Write_4_Bit('.');
    LCD_Write_4_Bit(b[0]); LCD_Write_4_Bit(b[1]);*/
}
//-------------------------------
//        Set cursor position
//-------------------------------
void SetPos(uchar position)
{
    LCD_FunctionMode(); 
    LCD_Write_4_Bit(position | 0x80); //
    LCD_DataMode();
}
//-------------------------------
//        Print string
//-------------------------------    
void PrintString(uchar *mess, uchar Position)
{
    SetPos(Position);
    while (*mess!='\0')
    {
        LCD_Write_4_Bit(*mess);
        mess++;
    }
}
//-------------------------------
//        Clear screen
//-------------------------------
void ClrScr(void)
{
    LCD_FunctionMode(); 
    LCD_Write_4_Bit(clear_lcd); 
    LCD_DataMode();
}
//-------------------------------
//        Wait for the response
//-------------------------------

void LCD_Wait(void)
{
    do
    {
       LCD_SEL=1;  
       LCD_RS=0;  LCD_WR=1;  LCD_DATA_7 = 1;//DAT=0xff;
    }  while(LCD_DATA_7 == 1);    
//    LCD_SEL=0;  LCD_WR=0;
}